;This file is copyright (c) 1991, 1992 Informant Communications Group and the
;article author. The material here may be used in an application provided
;that this copyright/disclaimer information is kept in the original source
;file. The material presented here is provided "as is" and with no guarantee.
;Informant Communications Group/Paradox Informant assume no responsibility
;for the use or misuse of the material contained within.
;
;Contents        : procedure AdHocQuery
;Source File     : ADHOCQUE.SC
;Author          : David Nassau
;                  Gnossos Software
;                  1534 16th Street, NW
;                  Washington, DC  20036
;                  (202) 387-0858
;
;Informant Issue : January 1992
;
;Description     : A set of procedures that allow for Ad Hoc querying of
;                  tables.
;
; Paradox Informant
; 8525 Elk Grove Blvd.
; Suite 126
; Elk Grove, CA  95624-1777
; Phone: (916) 686-6610
; Fax  : (916) 686-8497
; BBS  : (916) 686-4740


RELEASE PROCS ALL
RELEASE VARS ALL
CREATELIB "adhocque"

;____________________________________________________________________________

; AHPROMPT()
; Uses ?? command to place a prompt on the top two lines of the screen

PROC AHPROMPT(line1,line2)   ; custom prompts for entering data
    @ 1,0 ?? line2  + SPACES(80 - LEN(line2))
    @ 0,0 ?? line1 + SPACES(80 - LEN(line1))
    @ 0,LEN(line1)
ENDPROC
WRITELIB "adhocque" AHPROMPT

;____________________________________________________________________________

; AHMESSAGE()
; Displays message with blinking "..." following

PROC AHMESSAGE(messtext)
    msg = " " + messtext
    Style Attribute SysColor(3) @ 22, 76 - len(msg) ?? msg
    Style Attribute SysColor(3) + 128  ?? "... "
    Style
    RETURN
ENDPROC
WRITELIB "adhocque" AHMESSAGE

;____________________________________________________________________________

; CLEARQUERY()
; Clears all query images off workspace

PROC CLEARQUERY()
    IF NIMAGES() > 0 THEN
        MOVETO 1
        WHILE IMAGETYPE() = "Query"
            CLEARIMAGE
        ENDWHILE
    ENDIF
ENDPROC
WRITELIB "adhocque" CLEARQUERY
;____________________________________________________________________________

; DELETETABLE()
; Deletes table after testing for its existence

PROC DELETETABLE(Tablename)
    IF ISTABLE(Tablename) THEN
        DELETE Tablename
    ENDIF
ENDPROC
WRITELIB "adhocque" DELETETABLE

;____________________________________________________________________________

; PRIVATETABLE()
; Renames table to private table name

PROC PRIVATETABLE(tablename,newname)
    IF ISTABLE(newname) THEN
        DELETE newname
    ENDIF
    PRIVTABLES newname
    RENAME tablename newname
ENDPROC
WRITELIB "adhocque" PRIVATETABLE

;____________________________________________________________________________

; DELDUPES()
; Scans through current table, eliminates duplicate records

PROC DELDUPES()
    PRIVATE numfields,lastrec,currentrec,dupe,ct
    MESSAGE "Checking for duplicates"
    SORT TABLE()
    numfields = NFIELDS(TABLE())
    COEDITKEY
    COPYTOARRAY lastrec
    DOWN
    WHILE NOT RECORDSTATUS("New")
        COPYTOARRAY currentrec
        dupe = true
        FOR ct FROM 1 TO numfields
            IF lastrec[ct] <> currentrec[ct] THEN
                dupe = false
                QUITLOOP
            ENDIF
        ENDFOR
        IF dupe THEN
            DEL
        ENDIF
        COPYTOARRAY lastrec
        DOWN
    ENDWHILE
    DO_IT!
    HOME
ENDPROC
WRITELIB "adhocque" DELDUPES

;____________________________________________________________________________

; ADHOCQUERY()
; allows use to perform a query on one or more tables
; parameters: scrname - script to play to set up query in some cases
;             tabname - where gnscriptname is blank, table to query

PROC ADHOCQUERY(tabname,scrname)
    PRIVATE prompt1,prompt2,nexttab,lutable,msg,lufields,counter,count2,
            currentfield,tablechecks,numfields,currentrow

    IF NOT ISTABLE(tabname) and ISBLANK(scrname) THEN
        MESSAGE UPPER(tabname) + " does not exist! Press any key"
        retval = GETCHAR()
        RETURN false
    ENDIF

    prompt1 = "Enter query conditions, press F2 to process query"
    prompt2 = "F1 lookup, F10 link, F8 remove, F9 store/retrieve, " +
              "F3/F4 move, Esc cancel"
    quertable = tabname
    WHILE true
        CLEAR
        AHMESSAGE("Setting up query")
        DELETETABLE("Answer")
        IF quertable = "Narrow" OR ISBLANK(scrname) THEN
            MENU {Ask} SELECT quertable
            CHECK
        ELSE
            IF NOT ISBLANK(scrname) THEN
                PLAY scrname
            ENDIF
        ENDIF
        MOVETO 1
        GnNumTables = NIMAGES()
        WHILE True
            WAIT TABLE
                PROMPT prompt1,prompt2
            UNTIL "F1","F2","F10","F3","F4","F8","ESC","DOWN","DEL"
            SWITCH

                CASE retval = "DOWN" :
                    IF IMAGETYPE() = "Query" THEN
                        MESSAGE "Just a moment"
                        numfields = NFIELDS(TABLE())
                        ARRAY tablechecks[numfields]
                        currentfield = FIELD()
                        CTRLHOME
                        FOR counter FROM 1 TO numfields   ; record checks
                            RIGHT
                            tablechecks[counter] = CHECKMARKSTATUS()
                        ENDFOR
                        CTRLHOME
                        DOWN
                        FOR counter FROM 1 TO numfields
                            RIGHT
                            IF NOT ISBLANK(CHECKMARKSTATUS()) THEN
                                QUITLOOP    ; next row already has been added
                            ENDIF
                            EXECUTE tablechecks[counter]
                            DITTO           ; duplicate first row
                        ENDFOR
                        MOVETO FIELD currentfield
                    ELSE    ; not a query image
                        DOWN
                    ENDIF

                CASE retval = "DEL" AND IMAGETYPE() = "Query" :
                    SYNCCURSOR
                    currentrow = ROW()
                    UP
                    SYNCCURSOR
                    IF ROW() = currentrow THEN
                        MESSAGE "Cannot delete the first line of a query form"
                        KEYPRESS GETCHAR()
                    ELSE            ; delete current row of query image
                        DOWN
                        DEL
                        UP
                    ENDIF

                CASE Retval = "F10" :
                        AHPROMPT("","") ; blank out prompt line
                        nexttab = "X"
                        WHILE True
                            AHPROMPT("Table: ",
                    "Enter name of table to link to query or ENTER for list of tables")
                            ACCEPT "A8" TO nexttab
                            IF nexttab = "X" THEN
                                QUITLOOP
                            ENDIF
                            IF ISBLANK(nexttab) THEN
                                SHOWTABLES           ; show menu of tables
                                    DIRECTORY()
                                    "Select a table to link to query"
                                    TO nexttab
                                IF nexttab = "Esc" THEN
                                    QUITLOOP
                                ENDIF
                            ENDIF
                            IF ISTABLE(nexttab) THEN  ; add query image
                                MENU {Ask} SELECT nexttab
                                QUITLOOP
                            ELSE
                                MESSAGE nexttab + " is not a table"
                                nexttab = "X"
                            ENDIF
                        ENDWHILE

                CASE Retval = "F3" : UPIMAGE

                CASE Retval = "F4" : DOWNIMAGE

                CASE Retval = "F8" :
                    IF IMAGENO() <= GnNumTables THEN
                        MESSAGE "Cannot delete a primary image"
                        SLEEP 1500 ; can't delete any image with checked fields
                    ELSE
                        CLEARIMAGE
                    ENDIF

                CASE Retval = "F2" :
                    MESSAGE "Processing"
                    DO_IT!
                    IF ISTABLE("Answer") THEN  ; query sucessfully processed
                        MOVETO 1
                        IF ISTABLE("Addquer") THEN    ; if an "Add" query
                            ADD "Addquer" "Answer"
                            DELDUPES()  ; delete dupe records
                            DELETE "Addquer"
                        ENDIF
                        IF quertable = "Narrow" THEN ; if a "Narrow" query
                            DELETE "Narrow"
                        ENDIF
                        QUITLOOP
                    ELSE       ; query did not successfully process
                        msg = WINDOW()
                        PROMPT prompt1,prompt2
                        ECHO NORMAL
                        ECHO OFF
                        SYNCCURSOR
                        MESSAGE msg     ; show Paradox error message
                        KEYPRESS GETCHAR()
                        PROMPT
                    ENDIF

                CASE Retval = "ESC" :
                    CLEARALL
                    CLEAR
                    DELETETABLE("Narrow")
                    DELETETABLE("Addquer")
                    RETURN false        ; false indicates cancellation

                CASE Retval = "F1" :    ; lookup values in table
                    AHPROMPT("","")
                    lutable = "X"
                    WHILE True
                        AHPROMPT("Table: ",
                "Enter name of table to view or ENTER for list of tables")
                        ACCEPT "A60" TO lutable
                        IF lutable = "X" THEN
                            QUITLOOP
                        ENDIF
                        IF ISBLANK(lutable) THEN
                            SHOWTABLES      ; display menu of tables
                                DIRECTORY()
                                "Select a table to view"
                                TO lutable
                            IF lutable = "Esc" THEN
                                QUITLOOP
                            ENDIF
                        ENDIF
                        IF ISTABLE(lutable) THEN
                            VIEW lutable    ; put the table on the workspace
                            QUITLOOP
                        ELSE
                            MESSAGE lutable + " is not a table"
                            lutable = "X"
                        ENDIF
                    ENDWHILE
            ENDSWITCH
        ENDWHILE

        MOVETO 1
        WHILE TABLE() <> "Answer"
            CLEARIMAGE  ; clear all images except the Answer
        ENDWHILE

        IF NOT ISEMPTY("Answer") THEN
            WHILE true
                ECHO NORMAL  ; update canvas with current workspace
                ECHO OFF
                MESSAGE "Answer table has " + STRVAL(NRECORDS("Answer")) + " records"
                SHOWMENU
                    "Proceed"   :   "Continue operation: Answer is OK",
                    "Cancel"    :   "Cancel operation",
                    "Browse"    :   "View Answer before proceeding, delete records by hand",
                    "Narrow"    :   "Query on Answer table to narrow down " +
                                    "selected records",
                    "Add"       :   "Do another query, combine results with this Answer",
                    "Retry"     :   "Start query over from the beginning"
                TO choice
                SWITCH
                    CASE choice = "Proceed" :
                        CLEAR
                        RETURN true  ; true indicates that query processed OK

                    CASE choice = "Cancel" :
                        CLEARIMAGE
                        CLEAR
                        DELETETABLE("Narrow")
                        DELETETABLE("Addquer")
                        RETURN false    ;indicates cancellation

                    CASE choice = "Browse" :
                        EDITKEY
                        IMAGERIGHTS READONLY
                        WHILE true
                            WAIT TABLE
                                PROMPT "DEL to delete record, F2 to save",
                                       "ESC to cancel"
                            UNTIL "F2","DEL","ESC"
                            SWITCH
                                CASE retval = "DEL" :
                                    SHOWMENU
                                        "Delete" : "Delete current record",
                                        "Cancel" : "Do not delete record"
                                    TO delchoice
                                    IF delchoice = "Delete" THEN
                                        IMAGERIGHTS
                                        DEL
                                        IMAGERIGHTS READONLY
                                    ENDIF
                                CASE retval = "F2" :
                                    DO_IT!
                                    QUITLOOP
                                CASE retval = "ESC" :
                                    CANCELEDIT
                                    QUITLOOP
                            ENDSWITCH
                        ENDWHILE

                    CASE choice = "Narrow" :
                        PRIVATETABLE("Answer","Narrow")  ; rename Answer to
                        quertable = "Narrow"             ; priv. "Narrow"
                        CLEARQUERY()                     ; query on that
                        QUITLOOP

                    CASE choice = "Add" :
                        PRIVATETABLE("Answer","Addquer") ; rename Answer to
                        CLEARQUERY()                     ; private "Addquer"
                        quertable = tabname
                        QUITLOOP

                    CASE choice = "Retry" :
                        quertable = tabname  ; in case quertable was reset to
                        CLEAR                    ; "Narrow"
                        QUITLOOP
                ENDSWITCH
            ENDWHILE
        ELSE  ; answer is empty
            ECHO NORMAL
            ECHO OFF
            MESSAGE "No records were found"
            SHOWMENU
                "Cancel"    :   "Cancel querying",
                "Retry"     :   "Start query over again"
            TO choice
            IF choice = "ESC" OR choice = "Cancel" THEN
                CLEARIMAGE
                CLEAR
                DELETETABLE("Narrow")
                DELETETABLE("Addquer")
                RETURN false    ; indicates cancellation
            ELSE
                quertable = tabname
            ENDIF
        ENDIF
        CLEARIMAGE ;answer - for retry
        IF quertable <> "Narrow" AND ISASSIGNED(scrname)
                                        AND NOT ISBLANK(scrname) THEN
            PLAY scrname   ; sets up query image for "Add" or "Retry"
        ENDIF
    ENDWHILE
ENDPROC
WRITELIB "adhocque" ADHOCQUERY

autolib = "adhocque"
